_G.MOTD = MOTD or {}

MOTD._path = ModPath

MOTD._input_path = SavePath .. "motd.txt"

MOTD._options_path = ModPath .. "menu/options.txt"

MOTD._message = nil

MOTD.queued = {}

MOTD.settings = {
	enabled = true,
	randomized = false
}

function MOTD:IsEnabled()
	return self.settings.enabled
end

function MOTD:IsRandomized()
	return self.settings.randomized
end

function MOTD:SendMOTDByID(peer_id) --deprecated
	local peer = peer_id and managers.network:session():peer(peer_id)
	if peer then
		self:SendMOTDToPeer(peer)
		self.queued[peer_id] = false
	end
end

function MOTD:SendMOTDByID64(id)
	for _,peer in pairs(managers.network:session():peers()) do 
		if peer:user_id() == id then 
			if self:SendMOTDToPeer(peer) then
				self:SetQueuedPeer(peer,false)
			end
			return
		end
	end
end

function MOTD:SetQueuedPeer(peer,state)
	if peer then 
		self.queued[peer:user_id()] = state
	end
end

function MOTD:IsPeerQueued(peer)
	return peer and self.queued[peer:user_id()]
end

function MOTD:SendMOTDToPeer(peer)
	if self:IsEnabled() and peer and managers.chat then 
		local function send_message(s,_peer)
--			s = self:ApplyMessageMacros(s,{peer = peer})
			_peer:send("send_chat_message", managers.chat._channel_id or 1, s)
	--	peer:send_after_load("send_chat_message", managers.chat._channel_id or 1, s)
			managers.chat:receive_message_by_peer(managers.chat._channel_id or 1,managers.network:session():local_peer(),s,Color(0.7,0.1,0.6))
		end
		if self:IsRandomized() then
			local message = self._message[math.random(#self._message)]					
			if message then
				send_message(message,peer)
				return true
			end
		else
			local done_any
			for i,message in ipairs(self._message) do 
				send_message(message,peer)
				done_any = true
			end
			return done_any
		end
	end
	return false
end

function MOTD:ApplyMessageMacros(s,data) --not used
	data = data or {}
	if data.peer then 
		s = string.gsub(s,"$NAME",peer:name())
		s = string.gsub(s,"$LEVEL",peer:level())
		s = string.gsub(s,"$INFAMY",peer:rank())
	end
	return s
end

function MOTD:LoadMOTD()
	self._message = {}
	
	if not SystemFS:exists(Application:nice_path(self._input_path,true)) then 
		local file = io.open(self._input_path,"w+")
		file:close()
	end
	
	for line in io.lines(self._input_path) do
--		if line and (line ~= "") and (line ~= "\n") then
		table.insert(self._message,line)
--		end
	end
	return self._message
end

function MOTD:GetMOTD(formatted)
	local result = self._message
	if not self._message then 
		result = self:LoadMOTD()
	end
	
	if formatted then 
		for j,k in pairs(self._message) do
			self._message[j] = string.gsub(k,"\\n","\n")
		end
	end
	
	return result
end

function MOTD:SaveMOTD()
	local file = io.open(self._input_path,"w+")
	if file and self._message then
		file:write(table.concat(self._message,"\n"))
		file:close()
	end
end

function MOTD:LoadSettings()
	local file = io.open(self._options_path, "r")
	if (file) then
		for k, v in pairs(json.decode(file:read("*all"))) do
			self.settings[k] = v
		end
	else
		self:SaveSettings()
	end
end

function MOTD:SaveSettings()
	local file = io.open(self._options_path,"w+")
	if file then
		file:write(json.encode(self.settings))
		file:close()
	end
end

function MOTD:SetMOTD(tbl)
	if type(tbl) == "table" then 
		self._message = tbl
	else 
		self._message = {
			tostring(tbl)
		}
	end
	for j,k in pairs(self._message) do
		self._message[j] = string.gsub(k,"\\n","\n")
	end
	self:SaveMOTD()
	self:LoadMOTD()
end

MOTD:LoadMOTD()

Hooks:Add("LocalizationManagerPostInit", "MOTD_LocalizationManagerPostInit", function( loc )
	loc:add_localized_strings({
		motd_title = "Message of the Day",
		motd_set_enabled_title = "Toggle enabled",
		motd_set_enabled_desc = "Turn this mod on or off",
		motd_set_randomized_title = "Set random",
		motd_set_randomized_desc = "If true, selects one line from your MOTD at random to use when someone joins.",
		motd_set_new_title = "Set new MOTD",
		motd_set_new_desc = "Click to enter a new MOTD",
		motd_show_current_title = "Show current MOTD",
		motd_show_current_desc = "Click to show the current MOTD",
		motd_reload_title = "Reload MOTD...",
		motd_reload_desc = "Click to reload the MOTD from the save file",
		motd_current_title = "Current MOTD",
		motd_prompt_new_title = "Enter a new MOTD...",
		motd_prompt_new_desc = "Press Enter to Confirm, or press Escape to Cancel. Type \n for a newline.",
		motd_prompt_new_desc_1 = "******INSTRUCTIONS******:\nConfirm: Enter\nCancel: Escape\n\"\\n\" (without quotes) is a new line\n******CURRENT MOTD:*****\n", --not used
		motd_prompt_new_desc_2 = "\n************************\n\nType below:", --not used
		motd_menu_ok = "OK",
		error_mod_qki_missing_title = "You need to install 'Quick Keyboard Input'!",
		error_mod_qki_missing_desc = "In order to set your automated message from this in-game menu,\nplease download and install TdlQ's mod, 'Quick Keyboard Input', from here:\nhttp://pd2mods.z77.fr/quick_keyboard_input.html \nAlternatively\nyou can change the message by editing the file 'message.txt' located in this mod's folder."
	})
end)

Hooks:Add( "MenuManagerInitialize", "MOTD_MenuManagerInitialize", function(menu_manager)
	MenuCallbackHandler.callback_motd_show_current = function(self)
		QuickMenu:new(managers.localization:text("motd_current_title"),table.concat(MOTD:GetMOTD(),"\\n"),{
			{
				text = managers.localization:text("motd_menu_ok")
			}
		},true)
	end
	
	MenuCallbackHandler.callback_motd_set_motd = function(self)
		if QuickKeyboardInput then 
			QuickKeyboardInput:new(managers.localization:text("motd_prompt_new_title"),managers.localization:text("motd_prompt_new_desc"),table.concat(MOTD:GetMOTD(),"\\n"),callback(MOTD,MOTD,"SetMOTD"),300,true)
--			QuickKeyboardInput:new(managers.localization:text("motd_prompt_new_title"),managers.localization:text("motd_prompt_new_desc_1") .. table.concat(MOTD:GetMOTD(),"\n") .. managers.localization:text("motd_prompt_new_desc_2"),table.concat(MOTD:GetMOTD(),"\\n"),callback(MOTD,MOTD,"SetMOTD"),300,true)
		else
			QuickMenu:new(managers.localization:text("error_mod_qki_missing_title"),managers.localization:text("error_mod_qki_missing_desc"),{
				{
					text = managers.localization:text("motd_menu_ok")
				}
			},true)
		end
	end
	
	MenuCallbackHandler.callback_motd_reload = function(self)
		MOTD:LoadMOTD()
	end
	
	MenuCallbackHandler.callback_motd_set_enabled = function(self,item)
		MOTD.settings.enabled = item:value() == "on"
		MOTD:SaveSettings()
	end
	
	MenuCallbackHandler.callback_motd_set_randomized = function(self,item)
		MOTD.settings.randomized = item:value() == "on"
		MOTD:SaveSettings()
	end
	
	MOTD:LoadSettings()
	
	MenuHelper:LoadFromJsonFile(MOTD._options_path, MOTD, MOTD.settings)
end)
